/* GTK based ISDN monitor : inspired by the old isdnbutton program 
 * written in 1996 by M. Gutschke.
 * 
 * Differences :
 * 
 * - GTK based, not Xtoolkit based
 * - monitors both /proc/net/route and /dev/isdninfo, which
 *   allows more detailed information about the different states
 * - distinguishes incoming (blue) and outgoing (green) connections
 * - does not run suid : start it from the DisplayManager*setup:
 *   script for gdm, kdm or xdm if you need to run as root.
 * 
 * Kees Lemmens, Dec. 2001
 * Email: C.W.J.Lemmens@its.tudelft.nl
 * 
 */

#define VERSION "1.1, Dec. 2001"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <linux/isdn.h>
#include <gtk/gtk.h>

#include "interface.h"
#include "main.h"

/* 0 : no connection, no available device(s)   (red)    */
/* 1 : no connection, available ippp device(s) (yellow) */
/* 2 : connection established, outgoing        (green)  */
/* 3 : connection established, incoming        (blue)   */

Settings settings; /* global structure */

int checkState(void)
{
#define BUFSZ 4096
   char buf[BUFSZ],temp[BUFSZ],*ptr;
   FILE *fp;
   int state;
   int i, num;
   
   state = 0;
   
   /* Check if any isdn net device is configured (assigned with a route) */
   if ((fp = fopen("/proc/net/route","r")) != NULL)
   {
      while (!feof(fp) && fgets(buf,sizeof(buf),fp))
      {
	 if ((ptr = strstr(buf,"ippp")) != NULL)
	 {
	    ptr += 4;   /* skip "ippp" */
	    
	    sscanf(ptr,"%d",&num);

	    for (i=settings.startdev; i<=settings.enddev; i++)
	    {
	       if (num == i)
	       {
		  state = 1;
		  break;
	       }
	    }
	 }
      }
      fclose(fp);
   }
   
   /* Check if any CONFIGURED isdn net device is active (connected) */
   if (state == 1 && (fp = fopen("/dev/isdninfo","r")) != NULL)
   {
      for(i=0; i<4 && !feof(fp); i++) /* skip idmap + chmap + drmap */
	fgets(buf,sizeof(buf),fp);    /* and read usage field */
      fclose(fp);
      
      if ((ptr = strstr(buf,"usage")) != NULL)
      {	 
	 ptr += 7;                           /* skip tag + colon + space */
	 for (i=0; i<ISDN_MAX_CHANNELS; i++) /* skip non-monitored ippp devs */
	 {
	    sscanf(ptr,"%d",&num);
	    sscanf(ptr,"%s",temp);
	    ptr += strlen(temp) + 1;
	    
	    if ((num&ISDN_USAGE_MASK) == ISDN_USAGE_NET)
	    {
	       if (num&ISDN_USAGE_OUTGOING)
		 state = 2;
	       else
		 state = 3;
	    }
	 }
      }
   }
   settings.state = state;
   return state;
}

gint closeApp(GtkWidget *widget, GdkEvent *event, gpointer data )
{  
   gtk_main_quit();
   return(FALSE);
}

gint stateMonitor(gpointer data)
{
   int state = checkState();
   switchStateIcon(state);
/*   fprintf(stderr,"state : %d\n",state); */
   return TRUE; /* FALSE stops periodical execution of this function */
}

void usage(char *basename)
{
   fprintf(stderr,"\nUsage: %s [opts]\n",basename);
   fprintf(stderr,  "Version %s, (c) Kees Lemmens\n\n",VERSION);
   fprintf(stderr,  "Opts: -u <cmd>  : network up     cmd (%s)\n",NETUPCMD);
   fprintf(stderr,  "      -d <cmd>  : network down   cmd (%s)\n",NETDOWNCMD);
   fprintf(stderr,  "      -h <cmd>  : network hangup cmd (%s)\n",HANGUPCMD);
   fprintf(stderr,  "      -l <cmd>  : set label text (%s)\n",settings.label);
   fprintf(stderr,  "      -g <+x+y> : initital desktop position (%+d%+d)\n",
	   settings.xpos, settings.ypos);
   fprintf(stderr,  "      -s <nr>   : startnr device range to monitor (%d)\n",
	   settings.startdev);
   fprintf(stderr,  "      -e <nr>   : endnr   device range to monitor (%d)\n",
	   settings.enddev);
   exit(1);
}

void initialize(int argc, char *argv[])
{  
   char *basename;
   int x;

   basename=argv[0];
   
   /* Set default command strings : */
   settings.netup    = NETUPCMD;
   settings.netdown  = NETDOWNCMD;
   settings.hangup   = HANGUPCMD;
   settings.label    = "ISDN";
   settings.xpos     = settings.ypos = 0;
   settings.startdev = 0;
   settings.enddev   = ISDN_MAX_CHANNELS;
      
   x=argc;
   while(--x>0)                    /* parse options */
   {
      if(*argv[1]=='-')
      {
	 switch(*(++argv[1]))
	 {
	  case 'u': if(argc <= 2) usage(basename);
	    settings.netup=argv[2];
	    ++argv; --argc; --x; break;
	  case 'd': if(argc <= 2) usage(basename);
	    settings.netdown=argv[2];
	    ++argv; --argc; --x; break;
	  case 'h': if(argc <= 2) usage(basename);
	    settings.hangup=argv[2];
	    ++argv; --argc; --x; break;
	  case 'l': if(argc <= 2) usage(basename);
	    settings.label=argv[2];
	    ++argv; --argc; --x; break;
	  case 'g': if(argc <= 2) usage(basename);
	    sscanf(argv[2],"%d%d",
		   &settings.xpos,&settings.ypos);
	    ++argv; --argc; --x; break;
	  case 's': if(argc <= 2) usage(basename);
	    sscanf(argv[2],"%d",&settings.startdev);
	    ++argv; --argc; --x; break;
	  case 'e': if(argc <= 2) usage(basename);
	    sscanf(argv[2],"%d",&settings.enddev);
	    ++argv; --argc; --x; break;
	  default:  usage(basename); break;
	 }
	 ++argv; --argc;
      }
   }
}

int main (int argc, char *argv[])
{
   GtkWidget *top;

   gtk_init (&argc, &argv);
   initialize(argc,argv);
   
   top = createTopwindow();
   gtk_widget_set_uposition (top, settings.xpos, settings.ypos);
   gtk_signal_connect(GTK_OBJECT (top), "delete_event",
		      GTK_SIGNAL_FUNC (closeApp), NULL );
   
   gtk_widget_show(top);
   
   gtk_timeout_add(1500,stateMonitor,NULL);
   
   stateMonitor(NULL); /* set an initial state */
   gtk_main();
   return 0;
}

